==> suexec_helper-1.0.tar.gz: suexec_helper-1.0/mod_suexec_helper.c <==
/*
 * mod_suexec_helper - finer-grained control of suexec
 * Copyright (C) 2005 James Carter <james@jfc.org.uk>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include <stdio.h>
#include <pwd.h>
#include <grp.h>
#include <sys/types.h>

#include "httpd.h"
#include "http_config.h"
#include "http_protocol.h"
#include "http_request.h"
#include "http_log.h"
#include "apr_strings.h"
#include "unixd.h"


module AP_MODULE_DECLARE_DATA suexec_helper_module;


typedef struct _PerDirConfig {
    char *path;
    char *user;
    char *group;
} PerDirConfig;

static const command_rec commands[] = {
    AP_INIT_TAKE12("UserId", ap_set_string_slot,
        (void *) APR_XtOffsetOf(PerDirConfig, user),
        OR_LIMIT, "User name to exec scripts as"),
    AP_INIT_TAKE12("GroupId", ap_set_string_slot,
        (void *) APR_XtOffsetOf(PerDirConfig, group),
        OR_LIMIT, "Group name to exec scripts as"),
    {NULL}
};


static void *
create_per_dir_config(apr_pool_t *pool, char *path) {
    PerDirConfig *pdc = (PerDirConfig *)
        apr_pcalloc(pool, sizeof(struct _PerDirConfig));

    pdc->path = apr_pstrdup(pool, path);
    return pdc;
}

static ap_unix_identity_t *
get_id(const request_rec *r) {
    PerDirConfig *pdc;
    ap_unix_identity_t *id = NULL;
    struct passwd *pw;
    struct group *gr;

    pdc = (PerDirConfig *)
        ap_get_module_config(r->per_dir_config, &suexec_helper_module);
    if (pdc && pdc->user && (pw = getpwnam(pdc->user))) {
        id = apr_pcalloc(r->pool, sizeof(ap_unix_identity_t));
        id->uid = pw->pw_uid;
        id->gid = pw->pw_gid;
        if (pdc->group && (gr = getgrnam(pdc->group))) {
            id->gid = gr->gr_gid;
        }
        return id;
    } else {
        return NULL;
    }
}


static void
register_hooks(apr_pool_t *p) {
    ap_hook_get_suexec_identity(get_id, NULL, NULL, APR_HOOK_MIDDLE);
}

module AP_MODULE_DECLARE_DATA suexec_helper_module = {
    STANDARD20_MODULE_STUFF, 
    create_per_dir_config,
    NULL,
    NULL,
    NULL,
    commands,
    register_hooks
};

==> suexec_helper-1.1.tar.gz: suexec_helper-1.1/mod_suexec_helper.c <==
/*
 * mod_suexec_helper - finer-grained control of suexec
 * Copyright (C) 2005 James Carter <james@jfc.org.uk>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include <stdio.h>
#include <pwd.h>
#include <grp.h>
#include <sys/types.h>

#include "httpd.h"
#include "http_config.h"
#include "http_protocol.h"
#include "http_request.h"
#include "http_log.h"
#include "apr_strings.h"
#include "unixd.h"

/*backwards compatibility to apache 2.0*/
#ifndef  APR_OFFSETOF
#define APR_OFFSETOF APR_XtOffsetOf
#endif

module AP_MODULE_DECLARE_DATA suexec_helper_module;


typedef struct _PerDirConfig {
    char *path;
    char *user;
    char *group;
} PerDirConfig;

static const command_rec commands[] = {
    AP_INIT_TAKE1("UserId", ap_set_string_slot,
        (void *) APR_OFFSETOF(PerDirConfig, user),
        OR_LIMIT, "User name to exec scripts as"),
    AP_INIT_TAKE1("GroupId", ap_set_string_slot,
        (void *) APR_OFFSETOF(PerDirConfig, group),
        OR_LIMIT, "Group name to exec scripts as"),
    {NULL}
};


static void *
create_per_dir_config(apr_pool_t *pool, char *path) {
    PerDirConfig *pdc = (PerDirConfig *)
        apr_pcalloc(pool, sizeof(struct _PerDirConfig));

    pdc->path = apr_pstrdup(pool, path);
    return pdc;
}

static ap_unix_identity_t *
get_id(const request_rec *r) {
    PerDirConfig *pdc;
    ap_unix_identity_t *id = NULL;
    struct passwd *pw;
    struct group *gr;

    pdc = (PerDirConfig *)
        ap_get_module_config(r->per_dir_config, &suexec_helper_module);
    if (pdc && pdc->user && (pw = getpwnam(pdc->user))) {
        id = apr_pcalloc(r->pool, sizeof(ap_unix_identity_t));
        id->uid = pw->pw_uid;
        id->gid = pw->pw_gid;
        if (pdc->group && (gr = getgrnam(pdc->group))) {
            id->gid = gr->gr_gid;
        }
        return id;
    } else {
        return NULL;
    }
}


static void
register_hooks(apr_pool_t *p) {
    ap_hook_get_suexec_identity(get_id, NULL, NULL, APR_HOOK_MIDDLE);
}

module AP_MODULE_DECLARE_DATA suexec_helper_module = {
    STANDARD20_MODULE_STUFF, 
    create_per_dir_config,
    NULL,
    NULL,
    NULL,
    commands,
    register_hooks
};
